/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */
#include <Controller.h>
#include "D22SampleSequencer.h"
#include "controllers/common/family/Families.h"
#include "PersistenceServices/DataFile/NexusDataFile.h"

namespace d22special {

const string D22SampleSequencer::TYPE = "d22_sample_sequencer";

D22SampleSequencer::D22SampleSequencer(const std::string& name) :
		::SampleSequencer(name, "") {
	registerFunction(TYPE);

	setFamily(family::SAMPLE_ENVIRONMENT);

	//init properties
	integrationTime.init(this, SAVE, "integration_time");
	scansToAverage.init(this, SAVE, "scan_average");
	useSpectro.init(this, SAVE, "useSpectro");
	spectroController.init(this, "spectro");

	m_started = false;
}

D22SampleSequencer::~D22SampleSequencer() {
}

void D22SampleSequencer::postConfiguration() {
	SampleSequencer::postConfiguration();
}

void D22SampleSequencer::execute(const string& command, bool blocking, bool logging) {
	spectroController->mode = QE65000Controller::SEQUENCE;

	if (command == START_COMMAND) {
		m_started = true;
	} else if (command == STOP_COMMAND) {
		m_started = false;
	}

	SampleSequencer::execute(command, blocking, logging);
	//Once the last count is done force to do another spectro measure
	if (command == START_COMMAND) {
		doLastMeasure();
		m_started = false;
	}
//	spectroController->mode = QE65000Controller::ALONE;
}

void D22SampleSequencer::updateProperty(ChangeAspect* aChangeAspect) {

	SampleSequencer::updateProgress(aChangeAspect);
	string propertyName = aChangeAspect->getStringParam();

	if (propertyName == "actual_cha") {
		int32 actualSlot = any_cast<int32>(m_sampleChanger->getValue(propertyName));
		spectroController->setPlotName(actualSlot);
		actualSlotNumber = actualSlot;

	} else if (propertyName == "status_message") {
		std::string countStatus = any_cast<std::string>(m_Count->getValue(propertyName));

		if (countStatus == "Idle" && m_started == true) { // only save spectro if count has been started from this controller
			getSpectro();
		}

	} else if (propertyName == "numor") {
		lastNumor = any_cast<int32>(m_Count->getValue(propertyName));

	}
}

void D22SampleSequencer::refreshSetValue(string propertyName, const boost::any aValue) throw (CannotSetValue) {
	SampleSequencer::refreshSetValue(propertyName, aValue);

}

void D22SampleSequencer::getSpectro() {
	spectroController->start();

	//Copy data in nexus file (remember it is delayed in one position)
	if (actualSlotNumber > 1) {
		int32 numor = any_cast<int32>(m_Count->getValue("numor"));
		NexusDataFile::appendTo(numor - 1, "NxD22SpecialServerFile.xml", "NxD22SpecialClientFile.xml");
	}
}

void D22SampleSequencer::doLastMeasure() {

	//Last measure = move sample (using sampleChanger) + get spectro + save data in nexus file
	Sender<ClientCommand>::begin();
	Sender<ClientCommand>::post(ClientCommand(m_sampleChanger, START_COMMAND));
	Sender<ClientCommand>::endIf();

	spectroController->start();

	int32 numor = any_cast<int32>(m_Count->getValue("numor"));

	NexusDataFile::appendTo(numor, "NxD22SpecialServerFile.xml", "NxD22SpecialClientFile.xml");

//        cout << "adding new data (theoretically) in data file: " << numor << " value of last numor: " << lastNumor << endl;
}

}

/*
 void D22SampleSequencer::postConfiguration() {
 SampleSequencer::postConfiguration();
 //Set up measuring mode in spectro controller
 //	spectroController->mode = QE65000Controller::SEQUENCE;

 registerUpdater(sampleChanger->slot, &D22SampleSequencer::updateActualSlot, this);
 registerUpdater(count->statusMessage, &D22SampleSequencer::updateStatusMessage, this);
 registerUpdater(count->numor, &D22SampleSequencer::updateNumor, this);
 registerUpdater(spectroController->integrationTime, &D22SampleSequencer::updateIntegrationTime, this);
 registerUpdater(spectroController->scansToAverage, &D22SampleSequencer::updateScansToAverage, this);

 }

 void D22SampleSequencer::updateActualSlot() {
 spectroController->setPlotName(sampleChanger->slot());
 actualSlotNumber = sampleChanger->slot();
 }

 void D22SampleSequencer::updateStatusMessage() {
 std::string countStatus = count->statusMessage();
 if (countStatus == "Idle") {
 getSpectro();
 }
 }

 void D22SampleSequencer::updateNumor() {
 lastNumor = count->numor();
 }

 void D22SampleSequencer::updateIntegrationTime() {
 integrationTime = spectroController->integrationTime();
 }

 void D22SampleSequencer::updateScansToAverage() {
 scansToAverage = spectroController->scansToAverage();
 }

 void D22SampleSequencer::start() {
 commandStatus.setRunning();
 spectroController->mode = QE65000Controller::SEQUENCE;
 d22::SampleSequencer::start();
 //Once the last count is done force to do another spectro measure
 doLastMeasure();
 commandStatus.setIdle();
 spectroController->mode = QE65000Controller::ALONE;
 }

 void D22SampleSequencer::stop() {
 commandStatus.setWarning();
 d22::SampleSequencer::stop();
 }

 void D22SampleSequencer::getSpectro() {
 //Check the sampleSequencer is running
 if (commandStatus.isRunning()) {
 spectroController->start();
 //Copy data in nexus file (remember it is delayed in one position)
 if (actualSlotNumber > 1 && commandStatus.isRunning()) {
 int32 numor = any_cast<int32>(count->getValue("numor"));
 cout << "appending data to nexus file! (1)" << numor - 1 << endl;
 NexusDataFile::appendTo(numor - 1, "NxD22SpecialServerFile.xml", "NxD22SpecialClientFile.xml");
 }
 }
 }

 void D22SampleSequencer::doLastMeasure() {
 if (commandStatus.isRunning()) {
 //Last measure = move sample (using sampleChanger) + get spectro + save data in nexus file
 sampleChanger->start();
 spectroController->start();

 int32 numor = any_cast<int32>(count->getValue("numor"));
 cout << "appending data to nexus file! (2) " << numor << endl;
 NexusDataFile::appendTo(numor, "NxD22SpecialServerFile.xml", "NxD22SpecialClientFile.xml");
 //	cout << "adding new data (theoretically) in data file: " << numor << " value of last numor: " << lastNumor << endl;
 }

 }


}*/

